#ifndef _INC_CONTINUOUSAREACARTOGRAM_
#define _INC_CONTINUOUSAREACARTOGRAM_

class TINConstruction{
public:
	void TIN_Make_in_P(T_vector &i, co_vector &coord, P_vector &P);
private:
	void Adjust_TIN_Flip(const int N_T_first, T_vector &T, co_vector &coord);
	void Circumcircle(int *Point_ID, coord2D *center, double *radius_sq, co_vector &coord);
	void Laplacian_Smoothing(const int N_T_First, T_vector &T, co_vector &coord, i_vector &Added_V_ID);
	void Out_Tri_Info(int *T_V, T_vector &T);
	void Out_Tri_Info(const int T_V_0, const int T_V_1, const int T_V_2, T_vector &T);
	void Steiner(const int N_T_first, co_vector &coord, T_vector &T, i_vector &Added_V_ID);
	void Tin_Make_wo_Centroid(T_vector &T, i_vector &tmp_P_V, co_vector &coord);
	bool Flip(int *tmp_P_V, co_vector &coord, T_vector &T);
	bool Overturn_Triangle(const int Target, coord2D coord_COM, Triangle_Info T, co_vector &coord);
	bool Point_in_Circle(const int Point_ID, const double radius_sq, coord2D circle, co_vector &coord);
	bool Tri_Share_Edge(const int v_ID_1, const int v_ID_2, const int T_ID, const int min_T_ID,
			const int max_T_ID, T_vector &T, int *found_T_ID, int *not_shared_V_ID);
};

class ContinuousAreaCartogramConstruction{
public:
	double TIN_Area(const double Error_desired, double *myu, co_vector &coord, T_vector &T, E_vector &E_Outer);
private:
	void Create_Sparse_Matrix(d_matrix &X, i_matrix &X_ID, int i, int j, double value);
	void AreaCarto_Expl_Matrix(d_matrix &X, i_matrix &X_ID, double *Y, co_vector &Edge_Vec, co_vector &coord,
			E_vector &E, const int X_row, const double myu, T_vector &T);
	void calc_E_vector(co_vector &coord, co_vector &Edge_vec, E_vector &E);
	void Expand_Area(double *Total_A, co_vector &Edge_vec, co_vector &coord, T_vector &T);
	void Renew_coord(int N, co_vector &new_coord, double *Y);
	void Set_New_Edge_ID(E_vector &E, Triangle_Info *tmp_T, const int E_Order, const int V_Order0, const int V_Order1);
	void Triangle_Area(co_vector &Edge_vec, T_vector &T);
	bool TIN_Topology(double *myu, co_vector &new_coord, co_vector &old_coord, co_vector &Edge_vec, E_vector &E, E_vector &E_Outer, T_vector &T, i_vector &calc_V_ID);
	bool TIN_TopologyCheck(co_vector &new_coord, E_vector &E_Outer, T_vector &T, i_vector &calc_V_ID);
	int Set_Edge_ID(E_vector &E, T_vector &T);
	double Error_Ratio(const double Total_Data, T_vector &T);
	double Squared_Errors(T_vector &T);
	double TIN_Area_main(const double Error_desired, double *myu, co_vector &calc, T_vector &T, E_vector &E_Outer, i_vector &calc_V_ID);
};

class ContinuousAreaCartogram_IO{
public:
	bool Input_Data(char *in_Data, char *in_Area, co_vector &coord, P_vector &P, V_vector &V);
	void Output_P_V(char *in_Area, co_vector &coord, P_vector &P, V_vector &V);
	void Output_TIN(char *in_Area, co_vector &coord, T_vector &T);
private:
	void Check_PV(P_vector &P, co_vector &coord);
	bool Input_Area(char *in_Area, i_map &P_ID, P_vector &P);
	bool Input_V_coord(char *in_Data, i_map &in_V, co_vector &coord, V_vector &V);
	bool Input_P_V(char *in_Data, i_map &in_P, P_vector &P, i_map &in_V, V_vector &V);
	bool Check_Clockwise(i_vector &tmp_P_V, co_vector &coord);
};

class ContinuousAreaCartogramUtility{
public:
	void calc_angle(const int i, const int N_V, co_vector &Edge, d_vector &e_d, d_vector &c_v, d_vector &s_v);
	void calc_edge_length(const int i, i_vector &P_V, const int N_V, co_vector &coord, co_vector &Edge, d_vector &e_d);
	void calc_sincos_ijk(const int i, const int j, const int k, co_vector &coord, double *sin, double *cos);
	void Search_Vert(const int V1, const int V2, int *found_V1, int *found_V2, T_vector &T);
	void Set_Edge_ID(P_vector &P, E_vector &E);
	void Set_OuterEdge_ID(P_vector &P, E_vector &E);
	void Set_Origin(V_vector &V, co_vector &coord);
	bool intersect(const int S0, const int E0, const int S1, const int E1, co_vector &coord);
	bool Point_In_ThreePoints(const coord2D Point, const coord2D P0, const coord2D P1, const coord2D P2);
	int ID_check(const int ID, const int N_V);
	int Search_Same_Edge(Edge_Info tmp_ID, E_vector &E);
	double calc_edge_ij_length(const int i, const int j, co_vector &coord);
	double calc_tri_area_w_sign(co_vector &coord, const Triangle_Info T);
	double tri_area_w_sign(const int p1, const int p2, const int p3, co_vector &coord);
	double tri_area_w_sign(const coord2D p1, const coord2D p2, const coord2D p3);
};

bool large_ID_Value(const ID_Value& refLeft, const ID_Value& refRight);
void GetDimension2D(coord2D& min,coord2D& max, co_vector& co);

//OANZX\ȁAʃXbhpbP[W
UINT mainLoop(LPVOID pParam);

class ContinuousAreaCartogram{
public:
	char* in_Data;
	double Error, Error_desired, myu;
	bool m_bFileReady;

	P_vector P;
    V_vector V;
	co_vector coord;
	T_vector T;
	E_vector E_Outer;
	ContinuousAreaCartogram_IO IO;
	ContinuousAreaCartogramUtility Util;
	ContinuousAreaCartogramConstruction Construction;
	TINConstruction TIN;

	ContinuousAreaCartogram(){
		Error_desired = 0.1;
		myu = 1.0;
		m_pWait = new CEvent(FALSE,TRUE);
		m_pKill = new CEvent(FALSE,FALSE);
		m_bThread = m_bSuspend = m_bFileReady = FALSE;
	};
	~ContinuousAreaCartogram(){
		delete m_pWait;
		delete m_pKill;
	};

	//XbhǗIuWFNg
	CEvent* m_pWait;	//蓮
	CEvent* m_pKill;	//
	CWinThread* m_pThread;
	BOOL m_bThread,m_bSuspend;
	CCriticalSection m_cs;
	bool m_bUpdate;
	//OXbhZ[tɎQƂł镡f[^
	P_vector __P;
    V_vector __V;
	co_vector __coord;
	T_vector __T;
	E_vector __E_Outer;
	//
	bool Read(LPCSTR pFolder);
	bool Write(LPCSTR pFolder);
	bool Start();
	bool Stop();
	bool Resume();
	bool Kill();
};


#endif /* _INC_CONTINUOUSAREACARTOGRAM_ */